'use strict';

const config = require('../lib/config');

const nonExistentConfigFileName = 'not-a-file.json';
const invalidConfigFileName = './tests/test-configs/unit-tests/invalid.pageanrc.json';
const emptyConfigFileName = './tests/test-configs/unit-tests/empty.pageanrc.json';
const emptyUrlsConfigFileName = './tests/test-configs/unit-tests/empty-urls.pageanrc.json';
const emptyUrlValuesConfigFileName = './tests/test-configs/unit-tests/empty-url-values.pageanrc.json';
const noTestSettingsConfigFileName = './tests/test-configs/unit-tests/no-test-settings.pageanrc.json';
const globalTestSettingsConfigFileName = './tests/test-configs/unit-tests/global-test-settings.pageanrc.json';
const testSpecificSettingsConfigFileName = './tests/test-configs/unit-tests/test-specific-settings.pageanrc.json';
const globalAndTestSpecificSettingsConfigFileName = './tests/test-configs/unit-tests/global-and-test-specific-settings.pageanrc.json';
const puppeteerNoTestSettingsConfigFileName = './tests/test-configs/unit-tests/puppeteer-no-test-settings.pageanrc.json';
const urlTypesConfigFileName = './tests/test-configs/unit-tests/url-types.pageanrc.json';
const allTestPropsConfigFileName = './tests/test-configs/unit-tests/global-and-test-specific-settings-test-props.pageanrc.json';
const allShorthandConfigFileName = './tests/test-configs/unit-tests/global-and-test-specific-settings-shorthand.pageanrc.json';

const validHtmlHintConfigFilename = './.htmlhintrc';
const nonExistentHtmlHintConfigFilename = './tests/.htmlhintrc';

describe('invalid config', () => {
    it('should throw if config file not found', () => {
        expect.assertions(1);
        expect(() => config(nonExistentConfigFileName, nonExistentHtmlHintConfigFilename)).toThrow('ENOENT');
    });

    it('should throw if config file is not JSON', () => {
        expect.assertions(1);
        expect(() => config(invalidConfigFileName, nonExistentHtmlHintConfigFilename)).toThrow('Unexpected end of JSON input');
    });

    it('should throw if no URLs specified in config file', () => {
        expect.assertions(1);
        expect(() => config(emptyConfigFileName, nonExistentHtmlHintConfigFilename)).toThrow('No test URLs specified');
    });

    it('should throw if empty URL array specified in config file', () => {
        expect.assertions(1);
        expect(() => config(emptyUrlsConfigFileName, nonExistentHtmlHintConfigFilename)).toThrow('No test URLs found');
    });

    it('should throw if URL object with no url property specified in config file', () => {
        expect.assertions(1);
        expect(() => config(emptyUrlValuesConfigFileName, nonExistentHtmlHintConfigFilename)).toThrow('Invalid configuration');
    });
});

const checkConfigAgainstSnapshot = (configFileName, htmlHintConfigFilename) => {
    const processedConfig = config(configFileName, htmlHintConfigFilename);
    expect(processedConfig).toMatchSnapshot();
};

describe('puppeteer settings', () => {
    it('should return config with puppeteer launch settings', () => {
        expect.assertions(1);
        checkConfigAgainstSnapshot(puppeteerNoTestSettingsConfigFileName, nonExistentHtmlHintConfigFilename);
    });

    it('should return config with undefined puppeteer launch settings', () => {
        expect.assertions(1);
        checkConfigAgainstSnapshot(noTestSettingsConfigFileName, nonExistentHtmlHintConfigFilename);
    });
});

// eslint-disable-next-line max-lines-per-function
describe('test settings', () => {
    it('should return config with default test settings', () => {
        expect.assertions(1);
        checkConfigAgainstSnapshot(noTestSettingsConfigFileName, nonExistentHtmlHintConfigFilename);
    });

    it('should return config with global test settings', () => {
        expect.assertions(1);
        checkConfigAgainstSnapshot(globalTestSettingsConfigFileName, nonExistentHtmlHintConfigFilename);
    });

    it('should return config with default and test-specific settings', () => {
        expect.assertions(1);
        checkConfigAgainstSnapshot(testSpecificSettingsConfigFileName, nonExistentHtmlHintConfigFilename);
    });

    it('should return config with default, global, and test-specific settings', () => {
        expect.assertions(1);
        checkConfigAgainstSnapshot(globalAndTestSpecificSettingsConfigFileName, nonExistentHtmlHintConfigFilename);
    });

    it('should return config with all url types properly parsed', () => {
        expect.assertions(6); // eslint-disable-line no-magic-numbers
        const processedConfig = config(urlTypesConfigFileName, nonExistentHtmlHintConfigFilename);
        // Case 1: Local file that exists.  Use regex since path change in different environments.
        expect(processedConfig.urls[0].url).toMatch(/file:\/\/\/.*\/tests\/test-cases\/consoleLog.html/);
        // Case 2: Local file that does not exist, so assume URL (add https://)
        expect(processedConfig.urls[1].url).toStrictEqual('https://tests/test-cases/bar.html');
        // Case 3: URL with no protocol (add https://)
        expect(processedConfig.urls[2].url).toStrictEqual('https://localhost/foo.html'); // eslint-disable-line no-magic-numbers
        // Case 4: URL with port and no protocol (add https://)
        expect(processedConfig.urls[3].url).toStrictEqual('https://localhost:3000/foo.html'); // eslint-disable-line no-magic-numbers
        // Case 5: URL with http protocol (unchanged)
        expect(processedConfig.urls[4].url).toStrictEqual('http://localhost/foo.html'); // eslint-disable-line no-magic-numbers
        // Case 6: URL with https protocol (unchanged)
        expect(processedConfig.urls[5].url).toStrictEqual('https://localhost/foo.html'); // eslint-disable-line no-magic-numbers
    });

    it('should return the same config with test shorthand and property declaration', () => {
        expect.assertions(1);
        const shorthandConfig = config(allShorthandConfigFileName, nonExistentHtmlHintConfigFilename);
        const testPropsConfig = config(allTestPropsConfigFileName, nonExistentHtmlHintConfigFilename);
        expect(shorthandConfig).toStrictEqual(testPropsConfig);
    });
});

describe('htmlhint settings', () => {
    it('should return valid htmlhint config if .htmlhintrc file exists', () => {
        expect.assertions(1);
        const processedConfig = config(globalAndTestSpecificSettingsConfigFileName, validHtmlHintConfigFilename);
        expect(processedConfig.htmlHintConfig).toMatchSnapshot();
    });

    it('should return empty htmlhint config if .htmlhintrc file does not exist', () => {
        expect.assertions(1);
        const processedConfig = config(globalAndTestSpecificSettingsConfigFileName, nonExistentHtmlHintConfigFilename);
        expect(processedConfig.htmlHintConfig).toMatchSnapshot();
    });
});
