'use strict';

const consoleLogger = require('ci-logger');
const testLogger = require('../lib/logger');
const { testResultStates } = require('../lib/testUtils');

const defaultUrl = 'https://somewhere.com';
const defaultTestName = 'should run a test';

// eslint-disable-next-line max-lines-per-function
describe('startUrlTests', () => {
    let logger;
    beforeEach(() => {
        logger = testLogger();
    });

    afterEach(() => {
        jest.restoreAllMocks();
    });

    it('should add URL results object to test results', () => {
        expect.assertions(3); // eslint-disable-line no-magic-numbers
        // Console logger is mocked to avoid writing output to the console during the test
        // eslint-disable-next-line no-unused-vars
        const consoleLoggerSpy = jest.spyOn(consoleLogger, 'log').mockImplementation(() => {});

        const startingResults = logger.getTestResults();
        expect(startingResults.results).toHaveLength(0);

        logger.startUrlTests(defaultUrl);
        const endingResults = logger.getTestResults();
        expect(endingResults.results).toHaveLength(1);
        expect(endingResults.results[0].url).toStrictEqual(defaultUrl);
    });

    it('should log to console with URL being tested', () => {
        expect.assertions(1);
        const consoleLoggerSpy = jest.spyOn(consoleLogger, 'log').mockImplementation(() => {});
        logger.startUrlTests(defaultUrl);
        expect(consoleLoggerSpy).toHaveBeenCalledWith(expect.objectContaining({ message: expect.stringContaining(defaultUrl) }));
    });
});

// eslint-disable-next-line max-lines-per-function
describe('logTestResults', () => {
    let logger;
    beforeEach(() => {
        logger = testLogger();
    });

    afterEach(() => {
        jest.restoreAllMocks();
        delete require.cache[require.resolve('../lib/logger')];
    });

    // This includes only the console log from startUrlTests and getTestResults
    const consoleLoggerSpyDefaultCalls = 2;
    const testLogTestResults = (testResults) => {
        const consoleLoggerSpy = jest.spyOn(consoleLogger, 'log').mockImplementation(() => {});

        logger.startUrlTests(defaultUrl);
        logger.logTestResults(testResults);

        const allTestResults = logger.getTestResults();

        return { allTestResults, consoleLoggerSpy };
    };

    const testResultSymbols = Object.freeze({
        passed: '√',
        failed: '×',
        warning: '‼'
    });

    const testConsoleLogOutput = (testResult) => {
        const consoleSpy = jest.spyOn(console, 'log').mockImplementation(() => {});

        logger.startUrlTests(defaultUrl);

        const testResults = {
            name: defaultTestName,
            result: testResult,
            data: []
        };
        logger.logTestResults(testResults);
        return consoleSpy;
    };

    it('should not log test results if no results provided', () => {
        expect.assertions(2); // eslint-disable-line no-magic-numbers
        const { allTestResults, consoleLoggerSpy } = testLogTestResults();
        expect(allTestResults.results[0].tests).toHaveLength(0);
        expect(consoleLoggerSpy).toHaveBeenCalledTimes(consoleLoggerSpyDefaultCalls);
    });

    it('should log test results if provided', () => {
        expect.assertions(1);
        const testResults = {
            name: defaultTestName,
            result: testResultStates.passed,
            data: []
        };
        const { allTestResults } = testLogTestResults(testResults);
        expect(allTestResults.results[0].tests[0]).toStrictEqual(testResults);
    });

    it('should output test name to console if test results provided', () => {
        expect.assertions(1);
        const consoleSpy = testConsoleLogOutput(testResultStates.passed);
        // eslint-disable-next-line no-magic-numbers
        expect(consoleSpy).toHaveBeenNthCalledWith(2, expect.stringContaining(defaultTestName));
    });

    const testConsoleLogTestResults = (testResult) => {
        const consoleSpy = testConsoleLogOutput(testResult);
        /* eslint-disable no-magic-numbers */
        expect(consoleSpy).toHaveBeenNthCalledWith(2, expect.stringContaining(testResult));
        expect(consoleSpy).toHaveBeenNthCalledWith(2, expect.stringContaining(testResultSymbols[testResult]));
        /* eslint-enable no-magic-numbers */
    };

    it('should output test results to console for a passed test', () => {
        expect.assertions(2); // eslint-disable-line no-magic-numbers
        testConsoleLogTestResults(testResultStates.passed);
    });

    it('should output test results to console for a warning test', () => {
        expect.assertions(2); // eslint-disable-line no-magic-numbers
        testConsoleLogTestResults(testResultStates.warning);
    });

    it('should output test results to console for a failed test', () => {
        expect.assertions(2); // eslint-disable-line no-magic-numbers
        testConsoleLogTestResults(testResultStates.failed);
    });

    it('should increment only test and passed counters for a passed test', () => {
        expect.assertions(4); // eslint-disable-line no-magic-numbers
        const testResults = {
            name: defaultTestName,
            result: testResultStates.passed
        };
        const { allTestResults } = testLogTestResults(testResults);
        expect(allTestResults.summary.tests).toStrictEqual(1);
        expect(allTestResults.summary.passed).toStrictEqual(1);
        expect(allTestResults.summary.warning).toStrictEqual(0);
        expect(allTestResults.summary.failed).toStrictEqual(0);
    });

    it('should increment only test and failed counters for a failed test', () => {
        expect.assertions(4); // eslint-disable-line no-magic-numbers
        const testResults = {
            name: defaultTestName,
            result: testResultStates.failed
        };
        const { allTestResults } = testLogTestResults(testResults);
        expect(allTestResults.summary.tests).toStrictEqual(1);
        expect(allTestResults.summary.passed).toStrictEqual(0);
        expect(allTestResults.summary.warning).toStrictEqual(0);
        expect(allTestResults.summary.failed).toStrictEqual(1);
    });

    it('should increment only test and warning counters for a warning test', () => {
        expect.assertions(4); // eslint-disable-line no-magic-numbers
        const testResults = {
            name: defaultTestName,
            result: testResultStates.warning
        };
        const { allTestResults } = testLogTestResults(testResults);
        expect(allTestResults.summary.tests).toStrictEqual(1);
        expect(allTestResults.summary.passed).toStrictEqual(0);
        expect(allTestResults.summary.warning).toStrictEqual(1);
        expect(allTestResults.summary.failed).toStrictEqual(0);
    });
});

// eslint-disable-next-line max-lines-per-function
describe('getTestResults', () => {
    let logger;
    beforeEach(() => {
        logger = testLogger();
    });

    afterEach(() => {
        jest.restoreAllMocks();
    });

    const testGetTestResults = (includeTests, testName, testResult) => {
        const consoleLoggerSpy = jest.spyOn(consoleLogger, 'log').mockImplementation(() => {});

        if (includeTests) {
            logger.startUrlTests(defaultUrl);
            const name = testName;
            const result = testResult;
            logger.logTestResults({ name, result });
        }

        const results = logger.getTestResults();
        return {
            results,
            consoleLoggerSpy
        };
    };

    it('should return a results object with appropriate properties with no tests run', () => {
        expect.assertions(2); // eslint-disable-line no-magic-numbers

        const { results } = testGetTestResults(false);
        expect(results).toStrictEqual(expect.objectContaining({
            summary: {
                tests: 0,
                passed: 0,
                warning: 0,
                failed: 0
            },
            results: expect.any(Array)
        }));
        expect(results.results).toHaveLength(0);
    });

    it('should return a results object with appropriate properties with tests run', () => {
        expect.assertions(3); // eslint-disable-line no-magic-numbers

        const testName = defaultTestName;
        const testResult = testResultStates.passed;
        const { results } = testGetTestResults(true, testName, testResult);

        expect(results).toStrictEqual(expect.objectContaining({
            summary: {
                tests: 1,
                passed: 1,
                warning: 0,
                failed: 0
            },
            results: expect.any(Array)
        }));
        expect(results.results).toHaveLength(1);
        expect(results.results[0]).toStrictEqual(expect.objectContaining({
            url: defaultUrl,
            tests: [{ name: testName, result: testResult }] }));
    });

    it('should log to console the test result summary with total, passed, failed, and warning counts', () => {
        expect.assertions(4); // eslint-disable-line no-magic-numbers

        const testResult = testResultStates.warning;
        const { consoleLoggerSpy } = testGetTestResults(true, defaultTestName, testResult);

        // In this test function the third logged message is the summary.
        // eslint-disable-next-line prefer-destructuring, no-magic-numbers
        const { message } = consoleLoggerSpy.mock.calls[2][0];
        expect(message).toMatch(/Tests: 1/);
        expect(message).toMatch(/Passed: 0/);
        expect(message).toMatch(/Warning: 1/);
        expect(message).toMatch(/Failed: 0/);
    });
});
